@@ -1,10 +1,9 @@
-/* Copyright (C) 2015-2016 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
+/* Copyright (C) 2015-2017 Jason A. Donenfeld <Jason@zx2c4.com>. All Rights Reserved. */
 
 #ifndef WGCOOKIE_H
 #define WGCOOKIE_H
 
 #include "messages.h"
-#include "ratelimiter.h"
 #include <linux/rwsem.h>
 
 struct wireguard_peer;
@@ -13,9 +12,10 @@ struct sk_buff;
 
 struct cookie_checker {
 	u8 secret[NOISE_HASH_LEN];
+	u8 cookie_encryption_key[NOISE_SYMMETRIC_KEY_LEN];
+	u8 message_mac1_key[NOISE_SYMMETRIC_KEY_LEN];
 	u64 secret_birthdate;
 	struct rw_semaphore secret_lock;
-	struct ratelimiter ratelimiter;
 	struct wireguard_device *device;
 };
 
@@ -25,6 +25,8 @@ struct cookie {
 	u8 cookie[COOKIE_LEN];
 	bool have_sent_mac1;
 	u8 last_mac1_sent[COOKIE_LEN];
+	u8 cookie_decryption_key[NOISE_SYMMETRIC_KEY_LEN];
+	u8 message_mac1_key[NOISE_SYMMETRIC_KEY_LEN];
 	struct rw_semaphore lock;
 };
 
@@ -35,14 +37,15 @@ enum cookie_mac_state {
 	VALID_MAC_WITH_COOKIE
 };
 
-int cookie_checker_init(struct cookie_checker *checker, struct wireguard_device *wg);
-void cookie_checker_uninit(struct cookie_checker *checker);
+void cookie_checker_init(struct cookie_checker *checker, struct wireguard_device *wg);
+void cookie_checker_precompute_device_keys(struct cookie_checker *checker);
+void cookie_checker_precompute_peer_keys(struct wireguard_peer *peer);
 void cookie_init(struct cookie *cookie);
 
-enum cookie_mac_state cookie_validate_packet(struct cookie_checker *checker, struct sk_buff *skb, void *data_start, size_t data_len, bool check_cookie);
+enum cookie_mac_state cookie_validate_packet(struct cookie_checker *checker, struct sk_buff *skb, bool check_cookie);
 void cookie_add_mac_to_packet(void *message, size_t len, struct wireguard_peer *peer);
 
-void cookie_message_create(struct message_handshake_cookie *src, struct sk_buff *skb, void *data_start, size_t data_len, __le32 index, struct cookie_checker *checker);
+void cookie_message_create(struct message_handshake_cookie *src, struct sk_buff *skb, __le32 index, struct cookie_checker *checker);
 void cookie_message_consume(struct message_handshake_cookie *src, struct wireguard_device *wg);
 
 #endif
